
package require cgi
source htmllib.tcl
set fd [open_html_output $argv0]

cgi_eval {
    cgi_title "Using TCL and cgi.tcl to maintain your web page"

    cgi_body {

	std_margin {
	    TopicHeader "#FFFF99" "Introduction:"

	    p "This article will cover some basic TCL syntax, some cgi.tcl
	    usage and finally some examples and hints on how to get
	    the most from these tools."

	    p "TCL (pronounced 'tickle' or T-C-L) is a scripting language
	    available on a wide variety of platforms.  Written by John
	    Ousterhout, then maintained by Sun Microsystems, and currently
	    supported by Scriptics (www.scriptics.com), TCL is freely available
	    with source."

	    p "The latest OS/2 version of TCL is 7.6, ported by Illya
	    Vaes, is available on hobbes under /pub/os2/dev/tcl.  The
	    latest one I'm aware of is tk42os2x.zip.  This package includes a
	    textmode interpreter and a PM version which includes the graphical
	    toolkit, TK."

	    p "In addition to the base language, we'll also need cgi.tcl
	    from http://expect.nist.gov/cgi.tcl/ for this article.
	    cgi.tcl was written by Don Libes, also the author of
	    'expect', the invaluable tool for automating interactive
	    tasks (alas, this is only available for UNIX at the moment)."

	    p "A good description of cgi.tcl is from the homepage:"

	    p "[italic "cgi.tcl is the CGI support library for Tcl
	    programmers...  cgi.tcl can also be used for generating
	    static html (such as this page). This enables you to get
	    programming features into HTML, such as variables,
	    if/then/else, file I/O, etc."]"


	    TopicHeader "#FFFF99" "Installation:"

	    p "Follow the directions in the readme file of the TCL
	    package.  I've also renamed the text-mode program from
	    tclsh76.exe to tclsh.exe to make things follow my UNIX
	    experience more closely."

	    p "After installing the .exe and dll's and also setting
	    the [typewriter TCL_LIBRARY] variable to the correct
	    location, to test that everything is working:"

	    example {
		"\[OS/2] tclsh"
		"%"
	    }

	    p "You have a problem if you see the error:"
	    example {
		"application-specific initialization failed: Can't"
		"find a usable init.tcl in the following directories:"
		"\{\} \{\} ./lib/tcl7.6 I:/tcl7.6/library I:/library"
		"This probably means that Tcl wasn't installed properly."
	    }

	    TopicHeader "#FFFF99" "cgi.tcl"

	    p "cgi.tcl was intended for use in the unix environment,
	    but it works just fine under the OS/2 tcl environment.
	    Note that it available from the cgi.tcl page as a tar.Z or
	    a tar.gz format which means you'll need 'tar' from hobbes
	    as well as uncompress or gunzip.  After decompressing and
	    untarring the distribution, you may want to install cgi
	    package as a tcl package.  To do so:"

	    li "execute the tcl script pkgcreate which will create the
	    file pkgIndex.tcl"


	    li "Copy the two files to a location where tcl will look
	    for libraries.  On my installation, I installed the tcl
	    executables in /bin and set the variable
	    TCL_LIBRARY=h:/lib/tcl7.6/tcl7.6.  Then I created a lib
	    subdirectory under /lib/tcl7.6/tcl7.6/ and copied the two
	    file cgi.tcl and the package index to lib."

	    li "Test that the package is available by running the tcl
	    interpreter and loading the package."
	    example {
		"% package require cgi"
		"0.7.5"
		"% p \\\"test\\\""
		"<p>test</p>"
	    }

	    p "You may also want to copy the doc/ref.txt file which
	    has a list and description of all the cgi procedures available."


	    TopicHeader "#FFFF99" "Quick overview of TCL"

	    p "TCL is a wonderfully simple language which has a few
	    basic rules and consistant constructs, although if this is
	    your first exposure to the language, it may look quite odd."

	    li "TCL is a string based language.  Bourne shell
	    programmers will feel right at home, being able to execute the
	    contents of a variable if it is a valid command.  Everything is a
	    string, even numbers, there are no variable types or declarations."

	    p "Some examples of setting and using variables:"

	    example {
		"set command \\\"printf\\\""
		"set Name \\\"John Doe\\\""
		"set Age 50"
		"set user(age) $Age"
		"set user(name) $Name"
		"set elem id"
		"set user($elem) 501"

		"puts \\\"This is the equivalent of $command in TCL\\\""
		"puts \\\"Username = $user(name)\\\""
		"puts \\\"User age = $user(age) Userid = $user(id)\\\""
	    }

	    li "Functions in TCL are called procedures and are declared thus:"

	    example {
		"proc ProcedureName \\\{ args \\\} \\\{"
		" puts \\\"$args\\\""
	        "\\\}"
	    }
    
	    p "Notice the first set of brackets.  These are required
	    even if the procedure does not take any arguments (empty brackets).
	    Also important is the placement of the second left
	    bracket.  It must be on the first line, because it allows the
	    interpreter to continue to the next line to finish reading the
	    procedure."

	    p "To call a procedure, simply use the procedure name with any
	    required arguements."

	    example {"Procedure item1 \\\"item 2\\\" \\\{item 3\\\}"}

	    p "Note that the first argument is item1, the second
	    argument is surrounded by double quotes to keep the
	    interpreter from separating the item and 2.  The third
	    argument is surrounded by brackets for the same reason as
	    the second argument, but any variable substitution will
	    not be done in brackets as they would in quotes."
	    
	    p "Procedures can be called within constructs by using
	    square brackets:"

	    example {
		"proc1 arg1 \[proc2 arg1 arg2\] arg3"
	    }
	
	    p "More detailed coverage of TCL can be found throughout
	    the web.  References are at the end of this article."


	    TopicHeader "#FFFF99" "Using cgi.tcl"

	    p "Now that we have tcl working and cgi.tcl installed, we
	    can	write a simple tcl script that generates an html tag."

	    p "UNIX scripts have the ability to run the interpreter by
	    having '#!/path/to/binary' as the first line of the script.
	    Unfortunately, we don't have that ability.  As a workaround,
	    we either execute our scripts with \"tclsh scriptname.tcl\" or from
	    inside a script \"source scriptname.tcl\".  We cannot use the exec
	    command of TCL because it relies on the UNIX feature previously
	    mentioned."

	    p "Run the first example, we have a few tags generated:"
	    example {
		"<title>title</title>"
		"<body >"
		"<p>This article will cover some basic TCL syntax, some cgi.tcl"
		"usage and finally some examples and hints on how to get the most"
		"from these tools.</p>"
		"[...]"
		"</body>"
	    }

	    NOTE {At least on my system, after tclsh exits normally,
	    the queue to the command window seems to get locked.
	    i.e. keystrokes to the window are not registered.  If I
	    click on the title bar of the window, control returns.}

	    p "By default the cgi.tcl system will output to stdout
	    because of the way cgi works.  We can redirect standard
	    output into a file, examp1.html to capture the static
	    html.  Or with a little extra work, write a procedure that
	    will change the basic behavior of the cgi.tcl routines so that they
	    write directly to a file."


	    p "Executing examp2.tcl will not generate any output to
	    the screen (unless there are any errors) and will write
	    all the output into examp2.tcl without any manual redirection of
	    output at the command shell level."

	    p "Let's take a close look at what needs to be done to
	    implement this."

	    p "After the \"package requires cgi\" line, we load a tcl
	    file htmllib.tcl.  This contains a replacement function
	    for cgi_puts which is used by the rest of the cgi.tcl
	    library.  In order to replace it, we use the function (to
	    implicitly load it) and then use the rename procedure to
	    'move' the old proc out of the way and move ours in its place."

	    p "Another procedure in this 'library' is the
	    open_html_output procedure.  This takes a single argument
	    which opens that file for output and returns the file
	    descriptor of that file.  Generally we'll simply give the
	    name of the tcl source file as the argument which can be
	    found in the argv0 variable.  We can open different files
	    simultanously, if we are using frames for example."
	}
    }
}









