#include <glut.h>
#include "glWindow.h"
#include "glFile.h"

int menuId;
modelData *model = 0;

/* setup functions */
void initializeLighting()
{
   /* setup light 0 */
   GLfloat amb_light[] = { 0.1, 0.1, 0.1, 1.0 };
   GLfloat diffuse[] = { 0.6, 0.6, 0.6, 1 };
   GLfloat specular[] = { 0.7, 0.7, 0.3, 1 };

   glLightModelfv( GL_LIGHT_MODEL_AMBIENT, amb_light );
   glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuse );
   glLightfv( GL_LIGHT0, GL_SPECULAR, specular );
   glEnable( GL_LIGHT0 );
   glEnable( GL_COLOR_MATERIAL );

   glShadeModel( GL_SMOOTH );
   glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE );

   glDepthFunc( GL_LEQUAL );
   glEnable( GL_DEPTH_TEST );
   glEnable( GL_CULL_FACE );
}

void initializeTextureMapping()
{
   /* specify texture parameters */
   glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
   glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
   glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
   glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
   glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
}

int main( int argc, char *argv[] )
{
   int sub1, sub2, sub3;

   /* setup and create the window */
   glutInitWindowSize( 350, 400 );
   glutInit( &argc, argv );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   glutCreateWindow( "OpenGL Demo - Quake II Viewer" );

   initializeLighting();

   /* setup stuff in here */
   glutDisplayFunc( paintWindow );
   glutReshapeFunc( setWindowSize );
   glutIdleFunc( idleFunc );
   glutMouseFunc( mouseFunc );
   glutMotionFunc( mouseMotionFunc );

   /* create the menu */
   /* first sub menu */
   sub1 = glutCreateMenu( menuFuncDisplay );
   glutAddMenuEntry( "Points", MENU_SHOW_POINTS );
   glutAddMenuEntry( "Wireframe", MENU_SHOW_WIREFRAME );
   glutAddMenuEntry( "Solid", MENU_SHOW_SOLID );
   glutAddMenuEntry( "Texture", MENU_SHOW_TEXTURE );

   /* second sub menu */
   sub2 = glutCreateMenu( menuFuncLoad );
   glutAddMenuEntry( "Default", MENU_LOAD_DEFAULT );

   /* third sub menu */
   sub3 = glutCreateMenu( menuFuncMotionSelect );
   glutAddMenuEntry( "Rotate", MENU_MOTION_ROTATE );
   glutAddMenuEntry( "Zoom", MENU_MOTION_ZOOM );

   /* main menu */
   menuId = glutCreateMenu( menuFuncMain );
   glutAddSubMenu( "Display As...", sub1 );
   glutAddSubMenu( "Load Model...", sub2 );
   glutAddSubMenu( "Motion Type...", sub3 );
   glutAddMenuEntry( "Exit", MENU_EXIT );

   /* attach as popup to button */
   glutSetMenu( menuId );
   glutAttachMenu( GLUT_RIGHT_BUTTON );

   /* now load in the model from the parameter line */
   if( argc == 2 )
      model = loadModel( argv[1], 0 );
   else if( argc >= 3 )
      model = loadModel( argv[1], argv[2] );
   /* specify the texture loaded if any */
   initializeTextureMapping();
   if( model->texture )
   {
      glTexImage2D( GL_TEXTURE_2D, 0, 3, model->texWidth, model->texHeight, 0,
                                 GL_RGB, GL_UNSIGNED_BYTE, (void *)model->texture );
   }
   glutMainLoop();
   return( 0 );
}
