# this is a utility library for use with cgi.tcl.

package require cgi

# this procedure creates a file for writing, the only parameter passed to
# this is the name of the file to be created on output.  It will be stripped
# of the extension, if there is one, and a .html will be added to the name.
# This allows passing of $argv0 without processing on the caller's part.
# It returns the file descriptor returned by the open call.
proc open_html_output {arg1} {
  global argv0

  set filename [file rootname $arg1]
  return [open $filename.html w]
}

# A new puts function that uses the file descripter that should have been
# already set before the first output call.
proc new_cgi_puts {args} {
  global fd

  old_puts -nonewline $fd "[lindex $args end]"
  if {[llength $args] <= 1} {
     old_puts $fd ""
  }
}

rename puts old_puts
rename new_cgi_puts puts



# nice standard footer to use within the cgi_body command
proc std_footer {} {
    cgi_hr
    p "Last updated: [bold [exec date]]"
    p "ACME, Inc., Copyright [exec date +%Y], All Rights Reserved "
}

# A single wide table cell in a different color to create a separated
# topic header.  Also sets the anchor (a name) tag so we can reference it
# later.
proc TopicHeader { color args } {
    cgi_table BGCOLOR="$color" WIDTH="100%" {
        cgi_table_data {
            set args [join $args]
            puts "$args"
            while { [regsub {<[^>]*>} $args "" args] } {
            }
	    # this removes html tags so the anchor doesn't get confusing
            puts "[anchor_name $args]"
        }
    }
}
# sets up a table trick to get margins.  Maybe the margins should be settable
# through parameters? but then it wouldn't be standard would it?
proc std_margin { arg } {
    cgi_table WIDTH="100%" {
        cgi_table_data WIDTH="25%" {
            puts " "
        }
        cgi_table_data WIDTH="80%" {
                eval "$arg"
        }
        cgi_table_data WIDTH="5%" {
            puts " "
        }
    }
}

proc NOTE { args } {
    cgi_table BGCOLOR="#929292" WIDTH="70%" ALIGN=RIGHT {
       cgi_table_data BORDER=2 {
           puts "[bold NOTE:]"
       }
       cgi_table_data {
           puts "[join $args]"
       }
    }
}

proc example { args } {
    set args [join $args ]
    preformatted {
	foreach elem $args {
	    puts "[cgi_quote_html [join $elem]]"
	}
    }
}



